@@ -104,18 +104,26 @@ in_jlocaltime(const time_t* timep, struct jtm* result) {
 	struct tm t;
 	struct jtm c_jtm;
 	struct ab_jtm ab;
-	struct timeval tv;
-	struct timezone tz;
-
 	long int gmtoff;
 	time_t c;
 	
-	gettimeofday(&tv, &tz);
-	gmtoff = (-tz.tz_minuteswest) * J_MINUTE_LENGTH_IN_SECONDS +
-		(tz.tz_dsttime * J_HOUR_LENGTH_IN_SECONDS);
 	tzset();
 
 	localtime_r(timep, &t);
+
+#if defined _WIN32 || defined __MINGW32__ || defined __CYGWIN__
+	struct timeval tv;
+	struct timezone tz;
+
+	gettimeofday(&tv, &tz);
+	gmtoff = (-tz.tz_minuteswest) * J_MINUTE_LENGTH_IN_SECONDS +
+		(tz.tz_dsttime * J_HOUR_LENGTH_IN_SECONDS);
+	c_jtm.tm_zone = tzname[t.tm_isdst];
+#else
+	gmtoff = t.tm_gmtoff;
+	c_jtm.tm_zone = t.tm_zone;
+#endif
+
 	c = (*timep) + (time_t) gmtoff;
 
 	jalali_create_time_from_secs(c, &ab);
@@ -126,7 +134,6 @@ in_jlocaltime(const time_t* timep, struct jtm* result) {
 	c_jtm.tm_hour = ab.ab_hour;
 	c_jtm.tm_isdst = t.tm_isdst;
 
-	c_jtm.tm_zone = tzname[t.tm_isdst];
 	c_jtm.tm_gmtoff = gmtoff;
 
 	if (result) {
