@@ -34,6 +34,10 @@ extern const char* jalali_days_3[];
 extern const char* jalali_days_3_fa[];
 extern const char* jalali_days_2[];
 extern const char* jalali_days_2_fa[];
+extern const char* fa_jalali_months[];
+extern const char* fa_jalali_days_3[];
+extern const char* fa_jalali_days_2[];
+
 extern const int jalali_month_len[];
 
 extern char* optarg;
@@ -143,6 +147,8 @@ is_in_margin(struct cal_layout* l, struct cal_matrix* mat, int c) {
 void
 show_cal_matrix(struct cal_layout* l, struct cal_matrix* mat) {
 	int i, j, m;
+	char buf[100];
+
 	for (i=0; i<mat->height; i++) {
 		for (j=0; j<mat->width; j++) {
 			m = is_in_margin(l, mat, j);
@@ -153,33 +159,96 @@ show_cal_matrix(struct cal_layout* l, struct cal_matrix* mat) {
 				if (mat->m[i][j] == 0)
 					printf((l->julian) ? "    " : "   ");
 				else if (mat->m[i][j] > 1000 && l->color) {
-					printf((l->julian) ? "%s%3d%s " : "%s%2d%s ",
+					if (l->farsi) {
+						jalali_to_farsi(buf, 100, 2+l->julian,
+								 " ", mat->m[i][j] - 1000);
+					} else {
+						snprintf(buf, 100, "%*d",
+								 2+l->julian, mat->m[i][j] - 1000);
+					}
+
+					printf("%s%s%s ",
 						   TERM_BLACK_ON_WHITE,
-						   mat->m[i][j] - 1000, TERM_RESET);
+						   buf, TERM_RESET);
+
 				} else if (mat->m[i][j] > 1000 && !l->color) {
-					printf((l->julian) ? "%3d " : "%2d ", mat->m[i][j] - 1000);
+					if (l->farsi) {
+						jalali_to_farsi(buf, 100, 2+l->julian,
+								 " ", mat->m[i][j] - 1000);
+					} else {
+						snprintf(buf, 100, "%*d",
+								 2+l->julian, mat->m[i][j] - 1000);
+					}
+
+					printf("%s ", buf);
+
 				}
 				else {
-					printf((l->julian) ? "%3d " : "%2d ", mat->m[i][j]);
+					if (l->farsi) {
+						jalali_to_farsi(buf, 100, 2+l->julian,
+								 " ", mat->m[i][j]);
+					} else {
+						snprintf(buf, 100, "%*d",
+								 2+l->julian, mat->m[i][j]);
+					}
+
+					printf("%s ", buf);
 				}
 			}
 			else {
 				if (mat->m[i][j] == 0)
-					printf((l->julian) ? "   " : "	");
+					printf((l->julian) ? "   " : "  ");
 				else if (mat->m[i][j] > 1000 && l->color) {
-					printf((l->julian) ? "%s%3d%s" : "%s%2d%s",
+					if (l->farsi) {
+						jalali_to_farsi(buf, 100, 2+l->julian,
+								 " ", mat->m[i][j] - 1000);
+					} else {
+						snprintf(buf, 100, "%*d",
+								 2+l->julian, mat->m[i][j] - 1000);
+					}
+
+					printf("%s%s%s ",
 						   TERM_RED_ON_WHITE,
-						   mat->m[i][j] - 1000, TERM_RESET);
+						   buf, TERM_RESET);
+
 				} else if ((mat->m[i][j] > 1000) && (!l->color)) {
-					printf((l->julian) ? "%3d" : "%2d", mat->m[i][j] - 1000);
+					if (l->farsi) {
+						jalali_to_farsi(buf, 100, 2+l->julian,
+								 " ", mat->m[i][j] - 1000);
+					} else {
+						snprintf(buf, 100, "%*d",
+								 2+l->julian, mat->m[i][j] - 1000);
+					}
+
+					printf("%s", buf);
 				} else if (l->color) {
-					printf((l->julian) ? "%s%3d%s" : "%s%2d%s", TERM_RED,
-						   mat->m[i][j], TERM_RESET);
+					if (l->farsi) {
+						jalali_to_farsi(buf, 100, 2+l->julian,
+								 " ", mat->m[i][j]);
+					} else {
+						snprintf(buf, 100, "%*d",
+								 2+l->julian, mat->m[i][j]);
+					}
+
+					printf("%s%s%s",
+						   TERM_RED,
+						   buf,
+						   TERM_RESET);
+
 				}
 				else {
-					printf((l->julian) ? "%3d" : "%2d", mat->m[i][j]);
+					if (l->farsi) {
+						jalali_to_farsi(buf, 100, 2+l->julian,
+								 " ", mat->m[i][j]);
+					} else {
+						snprintf(buf, 100, "%*d",
+								 2+l->julian, mat->m[i][j]);
+					}
+
+					printf("%s", buf);
 				}
 			}
+			buf[0] = 0;
 		}
 		printf("\n");
 	}
@@ -220,7 +289,7 @@ destroy_cal_matrix(struct cal_matrix* mat) {
 
 
 /*
- * Displays calendarss, including cal title and week days.
+ * Displays calendar, including cal title and week days.
  */
 
 void
@@ -228,27 +297,46 @@ show_cal(struct cal_layout* l, struct cal_matrix* m, struct jtm** _j) {
 	char** ptr_d;
 
 	int i, k;
-	int cal_width = (l->julian) ? (7 * 3 + 6) : (7 * 2 + 6);
+	int cw = (l->farsi) ? 2 : 1;
+	int cal_width = (l->julian) ? 7 * 3 + 6 : 7 * 2 + 6;
 
 	char cal_t[3][MAX_BUF_SIZE];
-	char cal_y[3][20];
+	char cal_y[3][100];
 	int cal_tw[3];
-
-	if (l->julian)
-		ptr_d = (l->english) ? (char**)jalali_days_3 : (char**)jalali_days_3_fa;
+	char buf[100];
+	
+	if (l->farsi)
+		ptr_d = (l->julian) ? (char**) fa_jalali_days_3 :
+			(char**) fa_jalali_days_2;
+	else if (l->english)
+		ptr_d = (l->julian) ? (char**) jalali_days_3 : (char**) jalali_days_2;
 	else
-		ptr_d = (l->english) ? (char**)jalali_days_2 : (char**)jalali_days_2_fa;
+		ptr_d = (l->julian) ? (char**) jalali_days_3_fa :
+			(char**) jalali_days_2_fa;
+
 
 	for (i=0; i<m->n; i++) {
-		snprintf(cal_y[i], 20, "%d%s",
-				 _j[i]->tm_year + ((l->pahlavi) ? PAHLAVI_ISLAMIC_DIFF : 0),
-				 (l->pahlavi) ? "(pa)" : "");
+		if (l->farsi) {
+			jalali_to_farsi(buf, 100, 0, " ",
+					 _j[i]->tm_year + ((l->pahlavi) ?
+									   PAHLAVI_ISLAMIC_DIFF : 0));
+		} else {
+			snprintf(buf, 100, "%d",
+					 _j[i]->tm_year + ((l->pahlavi) ?
+									   PAHLAVI_ISLAMIC_DIFF : 0));
+		}
+		snprintf(cal_y[i], 100, "%s%s",
+				 buf,
+				 (l->pahlavi) ? ((l->farsi) ? " په" : "(pa)") : "");
+		buf[0] = 0;
 	}
 
 	for (i=0; i<m->n; i++) {
-		snprintf(cal_t[i], MAX_BUF_SIZE, "%s %s", jalali_months[_j[i]->tm_mon],
+		snprintf(cal_t[i], MAX_BUF_SIZE, "%s %s", (l->farsi) ?
+				 fa_jalali_months[_j[i]->tm_mon] :
+				 jalali_months[_j[i]->tm_mon],
 				 (l->syear) ? cal_y[i] : "");
-		cal_tw[i] = (cal_width - strlen(cal_t[i])) / 2;
+		cal_tw[i] = (cal_width - (strlen(cal_t[i]) / cw)) / 2;
 	}
 
 	for (i=0; i<m->n; i++) {
@@ -256,9 +344,12 @@ show_cal(struct cal_layout* l, struct cal_matrix* m, struct jtm** _j) {
 			printf(" ");
 		}
 
-		printf("%s", cal_t[i]);
+		printf("%s%s%s", TERM_WHITE, cal_t[i], TERM_RESET);
 
-		for (k=0; k<(cal_width	- cal_tw[i] - strlen(cal_t[i])); k++) {
+		for (k=0;
+			 k<(cal_width - cal_tw[i] -
+				(strlen(cal_t[i]) / cw + ((cw > 1) ? 1 : 0)));
+			 k++) {
 			printf(" ");
 		}
 
@@ -273,13 +364,13 @@ show_cal(struct cal_layout* l, struct cal_matrix* m, struct jtm** _j) {
 
 	for (i=0; i<m->n; i++) {
 		for (k=0; k<6; k++) {
-			printf("%s ", ptr_d[k]);
+			printf("%s%s%s ", TERM_WHITE, ptr_d[k], TERM_RESET);
 		}
-
+		
 		if (l->color)
 			printf("%s%s%s", TERM_RED, ptr_d[6], TERM_RESET);
 		else
-			printf("%s", ptr_d[6]);
+			printf("%s%s%s", TERM_WHITE, ptr_d[6], TERM_RESET);
 
 		if (i != m->n-1) {
 			for (k=0; k<l->margin; k++) {
@@ -377,7 +468,8 @@ void
 show_year(struct cal_layout* l, struct jtm* j) {
 	struct jtm _j[4];
 
-	char title[20];
+	char title[100];
+	char buf[100];
 
 	int cal_width = (((l->julian) ? (3 * 7 + 6) : (2 * 7 + 6)) * 3) +
 		(2 * l->margin);
@@ -385,9 +477,16 @@ show_year(struct cal_layout* l, struct jtm* j) {
 	int cal_tw;
 	int i;
 
-	snprintf(title, 20, "%d%s", j->tm_year + ((l->pahlavi) ?
-											  PAHLAVI_ISLAMIC_DIFF : 0),
-			 (l->pahlavi) ? "(pa)" : "");
+	if (l->farsi) {
+		jalali_to_farsi(buf, 100, 0, " ", j->tm_year + ((l->pahlavi) ?
+												 PAHLAVI_ISLAMIC_DIFF : 0));
+	} else {
+		snprintf(buf, 100, "%d", j->tm_year + ((l->pahlavi) ?
+												 PAHLAVI_ISLAMIC_DIFF : 0));
+	}
+
+	snprintf(title, 100, "%s%s", buf,
+			 (l->pahlavi) ? (l->farsi ? " پهلوی" : " (Pahlavi)") : "");
 
 	cal_tw = (cal_width - strlen(title)) / 2;
 
@@ -446,6 +545,7 @@ main(int argc, char** argv) {
 
 	l.color = 1;
 	l.pahlavi = 0;
+	l.farsi = 0;
 	l.julian = 0;
 	l.english = 0;
 	l.margin = 3;
@@ -489,6 +589,11 @@ main(int argc, char** argv) {
 			l.pahlavi = 1;
 			break;
 
+		case 'p':
+			l.farsi = 1;
+			l.english = 0;
+			break;
+
 			/* Displays Julian days (Day of year) instead of day of month.	*/
 		case 'j':
 			l.julian = 1;
